local addon = LibStub("AceAddon-3.0"):NewAddon("Archy", "AceConsole-3.0", "AceHook-3.0", "AceEvent-3.0", "AceTimer-3.0", "LibSink-2.0")
local ldb = LibStub("LibDataBroker-1.1"):NewDataObject("Archy", {
		label="Archy", 
		type="data source", 
		icon="Interface\\Icons\\Trade_Archaeology", 
		text="Archy!"
})
local icon = LibStub("LibDBIcon-1.0")

local DIGSITE_TEXTURE_INDEX = 177
local ARCHAEOLOGY_MAX_RACES = 12
local ARCHAEOLOGY_MAX_DISPLAY_RACES = 9
local ARCHAEOLOGY_MAX_DIGSITES_PER_CONTINENT = 4

local CONTINENTS = {}
CONTINENTS[1] = "Kalimdor"
CONTINENTS[2] = "Eastern Kingdoms"
CONTINENTS[3] = "Outlands"
CONTINENTS[4] = "Northrend"

local RACES = {}
RACES["Dwarf"] = 1
RACES["Draenei"] = 2
RACES["Fossil"] = 3
RACES["Night Elf"] = 4
RACES["Nerubian"] = 5
RACES["Orc"] = 6
RACES["Tol'vir"] = 7
RACES["Troll"] = 8
RACES["Vrykul"] = 9
RACES["Other"] = 10
RACES["UNKNOWN"] = 11
RACES["UNKNOWN"] = 12


local REFRESH_ON_NEXT_BAG_UPDATE = false
local LAST_DIGSITE = nil

local debugf = tekDebug and tekDebug:GetFrame("Archy")
local function Debug(...) if debugf then debugf:AddMessage(string.join(", ", tostringall(...))) else ChatFrame1:AddMessage(string.join(", ", tostringall(...))) end end
addon.Debug = Debug

function addon:OnInitialize()

	archyDB = archyDB or {}

	archyDB.digsites = archyDB.digsites or {}
	archyDB.frames = archyDB.frames or {}
	archyDB.frames.digsites = archyDB.frames.digsites or { position = { point="CENTER", x=0, y=0 } }
	archyDB.frames.artifacts = archyDB.frames.artifacts or { position = { point="CENTER", x=0, y=0 } }
	archyDB.icon = archyDB.icon or { hide = false }
	if archyDB.frames.show == nil then
		archyDB.frames.show	= true
	end

	self.frames = {
		digsites = {},
		artifacts = {},
	}

	self.digsites = {}
	self.artifacts = {}
	
	self.skill_rank = 0
	self.max_skill_rank = 0
	
	icon:Register("Archy", ldb, archyDB.icon)
	self:RegisterChatCommand("archy", "Command")

	self:ScheduleTimer("Refresh", 2)
end

function addon:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("WORLD_MAP_UPDATE")

	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE")
	self:RegisterEvent("SKILL_LINES_CHANGED")
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("BAG_UPDATE")

	self:RegisterEvent("ARTIFACT_DIG_SITE_UPDATED")
	self:RegisterEvent("ARTIFACT_HISTORY_READY")
	self:RegisterEvent("ARTIFACT_COMPLETE")
	self:RegisterEvent("ARTIFACT_UPDATE")

	-- setup digsites frame(s)
	self.frames.digsites.anchor = self:CreateDigsitesFrameAnchor()
	self.frames.rank = self:CreateRankFrame()
	self.frames.rank:SetPoint("TOPLEFT",self.frames.digsites.anchor,"TOPRIGHT",0,0)
	local prev = self.frames.rank
	for i = 1,ARCHAEOLOGY_MAX_DIGSITES_PER_CONTINENT do
		local dsFrame = self.CreateDigsiteFrame(i)
		if prev then
			dsFrame:SetPoint("TOPLEFT",prev,"BOTTOMLEFT",0,0)
		else
			dsFrame:SetPoint("TOPLEFT",self.frames.digsites.anchor,"TOPRIGHT",0,0)
		end
		
		self.frames.digsites[i] = dsFrame
		prev = dsFrame
	end

	if archyDB.frames.show then
		for i = 1,ARCHAEOLOGY_MAX_DIGSITES_PER_CONTINENT do
			self.frames.digsites[i]:Show()
		end
	end
	
	-- setup artifacts frame(s)
	self.frames.artifacts.anchor = self:CreateArtifactsFrameAnchor()
	prev = nil
	for i = 1,ARCHAEOLOGY_MAX_DISPLAY_RACES do
		local raFrame = self.CreateArtifactsFrame(i)
		if prev then
			raFrame:SetPoint("TOPLEFT",prev,"BOTTOMLEFT",0,0)
		else
			raFrame:SetPoint("TOPLEFT",self.frames.artifacts.anchor,"TOPRIGHT",0,0)
		end
		self.frames.artifacts[i] = raFrame
		prev = raFrame
	end
end

function addon:OnDisable()
end

function addon:Command(param)
	if param == "unlock" then
		self:Unlock()
	elseif param == "lock" then
		self:Lock()
	elseif param == "reset" then
		self:ResetFramePositions()
	elseif param == "refresh" then
		self:Refresh()
	elseif param == "toggle" then
		self:ToggleFrames()
	elseif param == "config" then
--		self.Debug("Config called")
	elseif param == "nearest" then
		self:AnnounceNearestDigsite()
	elseif param == "all" then
--		self.Debug("/archy all (NYI)")
	else
		self:Print("Usage:")
		self:Print(("%-20s"):format("/archy unlock"))
		self:Print(("%-20s"):format("/archy lock"))
		self:Print(("%-20s"):format("/archy reset"))
		self:Print(("%-20s"):format("/archy refresh"))
		self:Print(("%-20s"):format("/archy nearest"))
		self:Print(("%-20s"):format("/archy toggle"))
		self:Print(("%-20s"):format("/archy all (NYI)"))
		self:Print(("%-20s"):format("/archy config (NYI)"))
	end
end

function addon:ToggleFrames()
	archyDB.frames.show = not archyDB.frames.show
	self:ShowHideFrames(archyDB.frames.show)
end

function addon:ShowHideFrames(showFrames)
	if showFrames then 
		self.frames.rank:Show()
		for i,win in ipairs(self.frames.digsites) do
			win:Show()
		end
		for i,win in ipairs(self.frames.artifacts) do
			win:Show()
		end
	else
		self.frames.rank:Hide()
		for i,win in ipairs(self.frames.digsites) do
			win:Hide()
		end
		for i,win in ipairs(self.frames.artifacts) do
			win:Hide()
		end
	end
	if self.skill_rank == 525 then
		self.frames.rank:Hide()
	end
	
end

function addon:Unlock()
	self.frames.digsites.anchor:Show()
	self.frames.artifacts.anchor:Show()
end

function addon:Lock()
	self.frames.digsites.anchor:Hide()
	self.frames.artifacts.anchor:Hide()
end

function addon:ResetFramePositions()
	self.frames.digsites.anchor:SetPos("CENTER",0,0)
	self.frames.artifacts.anchor:SetPos("CENTER",0,0)
end

function addon:HasArchaeology()
	local _, _, arch = GetProfessions();
	return (arch ~= nil)
end

function addon:Refresh()
--	addon.Debug("HasArchaeology: ", self:HasArchaeology())
	if not self:HasArchaeology() then
		self:ShowHideFrames(false)
		return
	end
	self:UpdateDigsitesList()
	self:UpdateArtifactsList()
	self:ShowHideFrames(archyDB.frames.show)
end



function addon.CreateDigsiteFrame(id)
	local f = CreateFrame("Frame", nil, UIParent)
	local height = 20
	local width = 290
	f:SetWidth(width)
	f:SetHeight(height)
	f:SetFrameStrata("LOW")
	f.id = id
	
	-- zone
	f.zone = f:CreateFontString("zone", nil, "WatchFontTemplate")
	f.zone:SetWidth(160)
	f.zone:SetHeight(13)
	f.zone:SetJustifyH("RIGHT")
	f.zone:SetPoint("TOPLEFT",f,"TOPLEFT",1,1)
	f.zone:Show()

	-- race crest
	f.raceCrest = f:CreateTexture("raceCrest")
	f.raceCrest:SetWidth(14)
	f.raceCrest:SetHeight(14)
	f.raceCrest:SetPoint("TOPLEFT",f.zone,"TOPRIGHT",1,7)
	f.raceCrest:Show()

	f.raceTooltip = CreateFrame("Frame", nil, f)
	f.raceTooltip:SetSize(20,20)
	f.raceTooltip.digsiteID = id
	f.raceTooltip.fragmentType = "Unknown"
	f.raceTooltip:SetPoint("TOPLEFT",f.zone,"TOPRIGHT",1,7)
	f.raceTooltip:SetScript("OnEnter", function(self)
		if self.fragmentType ~= "UNKNOWN" then
			GameTooltip:SetOwner(self, "ANCHOR_NONE")
			GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
			GameTooltip:ClearLines()
			GameTooltip:AddLine(self.fragmentType .. " fragments are found at this digsite", 1, 1, 1)
			GameTooltip:Show()
		end
	end)
	f.raceTooltip:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	f.raceTooltip:Show()
	
	-- digsite name
	f.name = f:CreateFontString("name", nil, "WatchFontTemplate")
	f.name:SetWidth(200)
	f.name:SetHeight(13)
	f.name:SetJustifyH("LEFT")
	f.name:SetPoint("TOPLEFT",f.raceCrest,"TOPRIGHT",-15,-7)
	f.name:Show()
	
	return f
end

function addon.CreateArtifactsFrame(id)
	local f = CreateFrame("Frame", nil, UIParent)
	local height = 20
	local width = 290
	f:SetWidth(width)
	f:SetHeight(height)
	f:SetFrameStrata("LOW")
	f.id = id
	
	-- number of fragments available / total fragments needed for current artifact text frame
	f.progress = f:CreateFontString("progress", nil, "WatchFontTemplate")
	f.progress:SetWidth(50)
	f.progress:SetHeight(13)
	f.progress:SetJustifyH("RIGHT")
	f.progress:SetPoint("TOPLEFT",f,"TOPLEFT",1,1)
	f.progress:Show()

	f.sockets = f:CreateFontString("sockets", nil, "WatchFontTemplate")
	f.sockets:SetWidth(30)
	f.sockets:SetHeight(13)
	f.sockets:SetJustifyH("CENTER")
	f.sockets:SetPoint("TOPLEFT",f.progress,"TOPRIGHT",1,0)
	f.sockets:Show()

	-- race icon frame
	f.raceCrest = CreateFrame("BUTTON", nil, f, "UIPanelButtonTemplate")
	f.raceCrest.raceID = id
	f.raceCrest:SetWidth(14)
	f.raceCrest:SetHeight(14)
	f.raceCrest:SetPoint("TOPLEFT", f.sockets, "TOPRIGHT", 1, 7)
	f.raceCrest:RegisterForClicks("AnyUp")
	f.raceCrest:SetScript("OnEnter", function(self)
		if addon.artifacts[self.raceID] then
			local raceName = addon.artifacts[self.raceID].RaceInfo.Name
			GameTooltip:SetOwner(self, "ANCHOR_NONE")
			GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
			GameTooltip:ClearLines()
			GameTooltip:AddLine(raceName, 1, 1, 1)
			GameTooltip:AddLine("Left-Click to solve without keystones", 0, 1, 0)
			GameTooltip:AddLine("Right-Click to solve with keystones", 0, 1, 0)
			GameTooltip:Show()	
		end
	end)
	f.raceCrest:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	f.raceCrest:SetScript("OnClick", function(self, button, down)
		--ChatFrame1:AddMessage(button .. " Clicked")
		if button == "RightButton" then
			addon:SolveArtifact(self.raceID, true)
		else
			addon:SolveArtifact(self.raceID, false)
		end
	end)
	f.raceCrest:Show()
	
	-- race name/fragment type text frame
	f.fragmentType = CreateFrame("BUTTON", nil, f, "WatchFontTemplate")
	f.fragmentType.raceID = id
	f.fragmentType:SetSize(200,13)
--	f.fragmentType:SetJustifyH("LEFT")
	f.fragmentType:SetPoint("TOPLEFT",f.raceCrest,"TOPRIGHT",-1,-5)

	f.fragmentType:RegisterForClicks("AnyUp")	
	f.fragmentType:SetScript("OnClick", function(self, button, down)
		if addon.artifacts[self.raceID] then
			if addon.artifacts[self.raceID].Artifact then
				ShowUIPanel(ArchaeologyFrame)
				ArchaeologyFrame_ShowArtifact(self.raceID)
			end
		end
	end)
	f.fragmentType:SetScript("OnEnter", function(self)
		if addon.artifacts[self.raceID] then
			if addon.artifacts[self.raceID].Artifact then
				local raceName = addon.artifacts[self.raceID].RaceInfo.Name
				GameTooltip:SetOwner(self, "ANCHOR_NONE")
				GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
				GameTooltip:ClearLines()
				GameTooltip:AddLine("Left-Click to view this " .. raceName .. " artifact", 1, 1, 1)
				GameTooltip:Show()	
			end
		end
	end)
	f.fragmentType:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	f.fragmentType.text = f:CreateFontString("fragmentType", nil, "WatchFontTemplate")
	f.fragmentType.text:SetPoint("TOPLEFT", f.fragmentType,"TOPLEFT",0,0)
	f.fragmentType:Show()

	return f
end

function addon.CreateRankFrame(self)
	local f = CreateFrame("Frame", nil, UIParent)
	f:SetSize(290,20)
	f:SetFrameStrata("LOW")

	f.rank = f:CreateFontString("rank", nil, "WatchFontTemplate")
	f.rank:SetSize(290,13)
	f.rank:SetJustifyH("CENTER")
	f.rank:SetPoint("TOPLEFT",f,"TOPLEFT",1,1)
	f.rank:Show()

	return f
end

function addon.CreateDigsitesFrameAnchor(self)
	local f = CreateFrame("Frame",nil,UIParent)
	f:SetHeight(20)
	f:SetWidth(20)
	f.cols = cols
	f.filter = filter
	
	f:RegisterForDrag("LeftButton")
	f:EnableMouse(true)
	f:SetMovable(true)
	f:SetFrameStrata("HIGH")
	f:SetFrameLevel(2)
	
	local t = f:CreateTexture(nil,"BACKGROUND")
	t:SetTexture("Interface\\Buttons\\UI-RadioButton")
	t:SetTexCoord(0.25, 0.49, 0, 1)
	t:SetVertexColor(1, 0 ,0)
	t:SetAllPoints(f)
	
	f:SetScript("OnDragStart",function(self) self:StartMoving() end)
	f:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		_,_, archyDB.frames.digsites.position.point, archyDB.frames.digsites.position.x, archyDB.frames.digsites.position.y = self:GetPoint(1)
	end)
	
	f.SetPos = function(self,point,x,y)
		archyDB.frames.digsites.position.point = point
		archyDB.frames.digsites.position.x = x
		archyDB.frames.digsites.position.y = y
		self:ClearAllPoints()
		self:SetPoint(point, UIParent, point, x, y)
	end
	
	f:SetPos(archyDB.frames.digsites.position.point, archyDB.frames.digsites.position.x, archyDB.frames.digsites.position.y)
	
	f:Hide()
	
	return f
end

function addon.CreateArtifactsFrameAnchor()
	local f = CreateFrame("Frame",nil,UIParent)
	f:SetHeight(20)
	f:SetWidth(20)
	f.cols = cols
	f.filter = filter
	
	f:RegisterForDrag("LeftButton")
	f:EnableMouse(true)
	f:SetMovable(true)
	f:SetFrameStrata("HIGH")
	f:SetFrameLevel(2)
	
	local t = f:CreateTexture(nil,"BACKGROUND")
	t:SetTexture("Interface\\Buttons\\UI-RadioButton")
	t:SetTexCoord(0.25, 0.49, 0, 1)
	t:SetVertexColor(1, 0 ,0)
	t:SetAllPoints(f)
	
	f:SetScript("OnDragStart",function(self) self:StartMoving() end)
	f:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		_,_, archyDB.frames.artifacts.position.point, archyDB.frames.artifacts.position.x, archyDB.frames.artifacts.position.y = self:GetPoint(1)
	end)
	
	f.SetPos = function(self,point,x,y)
		archyDB.frames.artifacts.position.point = point
		archyDB.frames.artifacts.position.x = x
		archyDB.frames.artifacts.position.y = y
		self:ClearAllPoints()
		self:SetPoint(point, UIParent, point, x, y)
	end
	
	f:SetPos(archyDB.frames.artifacts.position.point, archyDB.frames.artifacts.position.x, archyDB.frames.artifacts.position.y)
	
	f:Hide()
	
	return f
end

function addon:IsValidContinent(continent)
	if continent >= 1 and continent <= 4 then
		return true
	end
	return false
end
function addon:UpdateDigsitesList()
	-- Refresh Digsites Frame
	self:UpdateWorldDigsites()
	
	local continent = GetCurrentMapContinent()
	if not self:IsValidContinent(continent) then
		return
	end
	for digsiteNum = 1,ARCHAEOLOGY_MAX_DIGSITES_PER_CONTINENT do
		if self.digsites[continent][digsiteNum] then
			local name = self.digsites[continent][digsiteNum].Name
			local zone = self.digsites[continent][digsiteNum].Zone
			local raceID = 0
			if archyDB.digsites and archyDB.digsites[name] then
				raceID = archyDB.digsites[name]
			end
			
			local RaceName, RaceCurrency, RaceTexture, RaceitemID = GetArchaeologyRaceInfo(raceID)
			
			self.frames.digsites[digsiteNum].name:SetText(name)
			self.frames.digsites[digsiteNum].zone:SetText(zone)
			self.frames.digsites[digsiteNum].raceTooltip.fragmentType = RaceName or "Unknown"
			self.frames.digsites[digsiteNum].raceCrest:SetTexture(RaceTexture)
			self.frames.digsites[digsiteNum].raceCrest:SetSize(44,44)
--			self.frames.digsites[digsiteNum].raceCrest:SetHeight(44)
		end
	end

	self:AnnounceNearestDigsite()
end



function addon:AnnounceNearestDigsite()
	if not self:HasArchaeology() then
		return
	end
	local continent = GetCurrentMapContinent()
	local nearestDigsite = self:NearestDigsiteName()
	local zoneOfNearestDigsite
	for digsiteNum = 1,ARCHAEOLOGY_MAX_DIGSITES_PER_CONTINENT do
		if self.digsites[continent][digsiteNum].Name == nearestDigsite then
			zoneOfNearestDigsite = self.digsites[continent][digsiteNum].Zone
		end
	end
	addon.Pour(addon, "Nearest digsite is: " .. nearestDigsite .. " in " .. zoneOfNearestDigsite, 1.0, 0.1, 0.1)
	addon:Print("Nearest digsite is: ", nearestDigsite, " in ", zoneOfNearestDigsite)
end

function addon:SolveArtifact(raceID, useKeystones)
	if self.artifacts[raceID] then
--		SetSelectedArtifact(raceID)
		self:UpdateRaceArtifact(raceID)
		
		-- Check Keystone Sockets
		if not self.artifacts[raceID].Artifact then
			addon:Print("No artifact available to solve for ", self.artifacts[raceID].RaceInfo.Name)
			return
		end
		if self.artifacts[raceID].Artifact.NumSockets > 0 then
			if useKeystones then
				for socketNum = 1,self.artifacts[raceID].Artifact.NumSockets do
					if ItemAddedToArtifact(self.artifacts[raceID].RaceInfo.KeystoneID) then
					else
						SocketItemToArtifact()
					end
				end
			elseif not useKeystones then
				for socketNum = 1,self.artifacts[raceID].Artifact.NumSockets do
					if ItemAddedToArtifact(self.artifacts[raceID].RaceInfo.KeystoneID) then
						RemoveItemFromArtifact()
					end
				end
			end
		end
		
		local base, adjust, totalCost = GetArtifactProgress();
		addon:Print("Trying to solve ", self.artifacts[raceID].RaceInfo.Name, " Artifact: ", 
						self.artifacts[raceID].Artifact.Name, " (Fragments: ", 
						min(base + adjust)," of ", totalCost, ")")
						
		if CanSolveArtifact() then
			SolveArtifact()
			REFRESH_ON_NEXT_BAG_UPDATE = true
		end
	end
end

function addon:UpdateAllArtifacts()
	for raceID = 1,ARCHAEOLOGY_MAX_DISPLAY_RACES do
		self:UpdateRaceArtifact(raceID)
	end
end

function addon:UpdateArtifactsList()
	-- Refresh Artifacts Frame
	self:UpdateAllArtifacts()
	for raceID = 1,ARCHAEOLOGY_MAX_DISPLAY_RACES do
		if self.artifacts[raceID] then
			local artifact = self.artifacts[raceID]
			self.frames.artifacts[raceID].raceCrest.raceID = raceID
			self.frames.artifacts[raceID].raceCrest:SetNormalTexture(artifact.RaceInfo.Texture)
			self.frames.artifacts[raceID].raceCrest:SetHighlightTexture(artifact.RaceInfo.Texture)
			self.frames.artifacts[raceID].raceCrest:SetPushedTexture(artifact.RaceInfo.Texture)
			self.frames.artifacts[raceID].raceCrest:SetDisabledTexture(artifact.RaceInfo.Texture)
			self.frames.artifacts[raceID].raceCrest:SetWidth(24)
			self.frames.artifacts[raceID].raceCrest:SetHeight(24)
			
			if artifact.Progress ~= nil and artifact.Artifact ~= nil then
				self.frames.artifacts[raceID].progress:SetText(min(artifact.Progress.Base+artifact.Progress.Adjust).."/"..artifact.Progress.TotalCost)
--			end
--			if artifact.Artifact ~= nil then
				if artifact.Artifact.NumSockets then
					if artifact.Artifact.NumSockets > 0 then
						self.frames.artifacts[raceID].sockets:SetText(artifact.Artifact.NumSockets)
					else
						self.frames.artifacts[raceID].sockets:SetText("")
					end
				end
				if artifact.Artifact.Rarity then
					if artifact.Artifact.Rarity == 0 then
						self.frames.artifacts[raceID].fragmentType.text:SetText(artifact.Artifact.Name)
					else
						self.frames.artifacts[raceID].fragmentType.text:SetText("|cFF0066FF" .. artifact.Artifact.Name)
					end
				end
			end

		end
	end
end

function addon:UpdateRaceArtifact(raceID)
	local RaceName, RaceCurrency, RaceTexture, RaceitemID = GetArchaeologyRaceInfo(raceID)
	local numProjects = GetNumArtifactsByRace(raceID)
	
	if numProjects == nil or numProjects == 0 then
		self.artifacts[raceID] = {
			RaceInfo = {
				Name = RaceName,
				Currency = RaceCurrency,
				Texture = RaceTexture,
				KeystoneID = RaceitemID
			},
			Progress = nil,
			Artifact = nil,
		}
	else
		local name, description, rarity, icon, spellDescription, numSockets, bgTexture =  GetActiveArtifactByRace(raceID)
		SetSelectedArtifact(raceID)
		local base, adjust, totalCost = GetArtifactProgress()
	--	local name, description, rarity, icon, spellDescription, numSockets, bgTexture =  GetSelectedArtifactInfo()
		
		
		self.artifacts[raceID] = {
			RaceInfo = {
				Name = RaceName,
				Currency = RaceCurrency,
				Texture = RaceTexture,
				KeystoneID = RaceitemID
			},
			Progress = {
				Base = base,
				Adjust = adjust,
				TotalCost = totalCost
			},
			Artifact = {
				Name = name,
				Description = description,
				Rarity = rarity,
				Icon = icon,
				SpellDescription = spellDescription,
				NumSockets = numSockets,
				BGTexture = bgTexture
			}
		}
	end
	
end

function addon:UpdateWorldDigsites()
	for k,v in pairs(CONTINENTS) do
		self:UpdateContinentDigsites(k)
	end
end

function addon:UpdateContinentDigsites(continentID)
	self.digsites[continentID] = {}
--	self.Debug("Continent ",continentID," ",CONTINENTS[continentID])
	SetMapZoom(continentID)
	local totalPOIs = GetNumMapLandmarks()
	local digsiteNum = 1
	for index = 1,totalPOIs do
		local worldMapPOIName = "WorldMapFramePOI"..index
		local worldMapPOI = _G[worldMapPOIName]
		if index <= totalPOIs then
			local name, description, textureIndex, px, py, mapLinkID, inBattleMap, graveyardID, areaID = GetMapLandmarkInfo(index)
			if textureIndex == DIGSITE_TEXTURE_INDEX then

				zoneName, fileName, texPctX, texPctY, texX, texY, scrollX, scrollY = UpdateMapHighlight(px, py)

				self.digsites[continentID][digsiteNum] = {
					Name = name,
					Zone = zoneName
				}
				digsiteNum = digsiteNum + 1
			end
		end
	end
	SetMapToCurrentZone()
end

function addon:NearestDigsiteName()
	SetMapZoom(GetCurrentMapContinent())
	local totalPOIs = GetNumMapLandmarks()
	local x,y = GetPlayerMapPosition("player")
	
	local digsiteName
	local distance
	
	for index = 1,totalPOIs do
		local worldMapPOIName = "WorldMapFramePOI"..index
		local worldMapPOI = _G[worldMapPOIName]
		if index <= totalPOIs then
			local name, description, textureIndex, px, py, mapLinkID, inBattleMap, graveyardID, areaID = GetMapLandmarkInfo(index)
			if textureIndex == DIGSITE_TEXTURE_INDEX then
				local distanceToDigsite = abs(self:DistanceBetweenPoints(x,y,px,py))
--				self.Debug("DISTANCE: " .. name .. " is " .. distanceToDigsite .. " away")

				if distance then
					if distanceToDigsite < distance then
						distance = distanceToDigsite
						digsiteName = name
					end
				else
					distance = distanceToDigsite
					digsiteName = name
				end
			end
		end
	end
	
	return digsiteName
end

function addon:DistanceBetweenPoints(x1,y1,x2,y2)
	return sqrt(((x2-x1)^2) + ((y2-y1)^2))
end

function addon:BAG_UPDATE()
	if REFRESH_ON_NEXT_BAG_UPDATE then
		self:UpdateArtifactsList()
		REFRESH_ON_NEXT_BAG_UPDATE = false
	end
end

function addon:WORLD_MAP_UPDATE()
end

function addon:PLAYER_ENTERING_WORLD()
	if not self:IsValidContinent(GetCurrentMapContinent()) then
		self:ShowHideFrames(false)
		return
	end

	self:Refresh()
end

function addon:SKILL_LINES_CHANGED()
	local _, _, arch = GetProfessions()
	if arch then
		local name, texture, rank, maxRank = GetProfessionInfo(arch)
		self.skill_rank = rank
		self.max_skill_rank = maxRank
		local msg = "               " .. rank .. "/" .. maxRank
		if rank >= (maxRank - 25) and maxRank < 525 then
			self.frames.rank.rank:SetText("|cFFFF0000" .. msg .. " - Visit a trainer")
		else
			self.frames.rank.rank:SetText(msg)
		end
		
		if self.skill_rank == 525 then
			self.frames.rank:Hide()
		end
	end
end

function addon:UNIT_SPELLCAST_SUCCEEDED(event, unit, spell, ...)
	if unit == "player" and spell == "Survey" then
		LAST_DIGSITE = self:NearestDigsiteName()
	--elseif unit == "player" and spell == "Searching for Artifacts" then
	end
end

function addon:CHAT_MSG_LOOT(event, msg)
	fragment_type, amount = strmatch(msg, "You receive currency: (.+) Archaeology Fragment x(%d+)")
	if fragment_type then
		local zone = GetZoneText()
		local subzone = GetSubZoneText()
		
		-- Add Digsite to DB if it hasn't been discovered yet
		if not archyDB.digsites[LAST_DIGSITE] then
			archyDB.digsites[LAST_DIGSITE] = RACES[fragment_type]
			self:UpdateDigsitesList()
		end
--		self.Debug("Looted ",amount,"x",fragment_type," fragments at ",LAST_DIGSITE," in ",subzone," (",zone,")")
	end
end

function addon:ARTIFACT_DIG_SITE_UPDATED()
	self:UpdateDigsitesList()
end

function addon:ARTIFACT_UPDATE()
end

function addon:ARTIFACT_COMPLETE()
	self:UpdateArtifactsList()
end

function addon:ARTIFACT_HISTORY_READY()
end

function addon:CURRENCY_DISPLAY_UPDATE()
	self:UpdateArtifactsList()
end

-- Data Broker --
function ldb:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_NONE")
	GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
	GameTooltip:ClearLines()

	GameTooltip:AddLine("Archy", 1, 1, 1)
	GameTooltip:AddLine("Left-Click to toggle Archy", 0, 1, 0)
	GameTooltip:AddLine("Right-Click to lock/unlock Archy", 0, 1, 0)
	GameTooltip:AddLine("Middle-Click to display the Archaeology window", 0, 1, 0)

	GameTooltip:Show()	
end

function ldb:OnLeave()
	GameTooltip:Hide()
end

function ldb:OnClick(button)
	if button == "LeftButton" then
		addon:ToggleFrames()
	elseif button == "RightButton" then
		if addon.frames.digsites.anchor:IsShown() then
			addon:Lock()
		else
			addon:Unlock()
		end
	elseif button == "MiddleButton" then
		ShowUIPanel(ArchaeologyFrame)
	end
end


